const EXPORTED_SYMBOLS = ['TabDockBookmarks'];

Components.utils.import("resource://gre/modules/Services.jsm");
Components.utils.import('resource://gre/modules/XPCOMUtils.jsm');

Components.utils.import('chrome://tabdock/content/tabdock-utils.js');

// TODO - Currently unable get bookmarks working with Services.bookmarks
var bookmarksService = bookmarksService || Components.classes["@mozilla.org/browser/nav-bookmarks-service;1"].getService(Components.interfaces.nsINavBookmarksService);

const Cc = Components.classes;
const Ci = Components.interfaces;

// An nsINavBookmarkObserver
var TabDockBookmarksListener = {
	onBeginUpdateBatch: function() {},
	onEndUpdateBatch: function() {},
	
	onItemAdded: function(aItemId, aFolder, aIndex) {
		if(aFolder === TabDockBookmarks.BOOKMARK_FOLDER) {
		    TabDockBookmarks.bookmarkAdded(aItemId, aFolder, aIndex);
	        TabDockBookmarks.bookmarkMoved(aItemId, aParentId, index);
		}
	},
	onItemRemoved: function(aItemId, aFolder, aIndex) {},
	onBeforeItemRemoved: function(aItemId, aItemType, aParentId, aGUID, aParentGUID) {
		if(aParentId === TabDockBookmarks.BOOKMARK_FOLDER) {
		    let index = bookmarksService.getItemIndex(aItemId);
			TabDockBookmarks.bookmarkRemoved(aItemId, aParentId, index);
		}
	},
	onItemChanged: function(aItemId, aProperty, aIsAnnotationProperty, aNewValue, 
			aLastModified, aItemType, aParentId, aGUID, aParentGUID) {
		
		if(aProperty === "uri" && aParentId === TabDockBookmarks.BOOKMARK_FOLDER) {
		    let index = bookmarksService.getItemIndex(aItemId);
			TabDockBookmarks.bookmarkRemoved(aItemId, aParentId, index);
			TabDockBookmarks.bookmarkAdded(aItemId, aParentId, index);
		    TabDockBookmarks.bookmarkMoved(aItemId, aParentId, index);
		}
	},
	onItemVisited: function(aBookmarkId, aVisitID, time) {},
	onItemMoved: function(aItemId, aOldParent, aOldIndex, aNewParent, aNewIndex) {
		if(aNewParent === aOldParent && aNewParent === TabDockBookmarks.BOOKMARK_FOLDER)
			TabDockBookmarks.bookmarkMoved(aItemId, aNewParent, aNewIndex);
		else if(aNewParent === TabDockBookmarks.BOOKMARK_FOLDER)
			TabDockBookmarks.bookmarkAdded(aItemId, aNewParent, aNewIndex);
		else if(aOldParent === TabDockBookmarks.BOOKMARK_FOLDER)
			TabDockBookmarks.bookmarkRemoved(aItemId, aOldParent, aOldIndex);
	},
	
	QueryInterface: XPCOMUtils.generateQI([Components.interfaces.nsINavBookmarkObserver])
};

var TabDockBookmarks = (function() {

	var _init = function() {
		TabDockUtils.dump("start init bookmarks\n");
		
		bookmarksService.addObserver(TabDockBookmarksListener, false);
		
		let historyService = Components.classes["@mozilla.org/browser/nav-history-service;1"].getService(Components.interfaces.nsINavHistoryService);
		let options = historyService.getNewQueryOptions();
	    let query = historyService.getNewQuery();
	    query.setFolders([bookmarksService.toolbarFolder], 1);
	    
	    _bookmarkFolder = historyService.executeQuery(query, options).root;
	    
		TabDockUtils.dump("done init bookmarks\n");
	}
 

	var _bookmarkFolder = null;
    var _eventSupport = new EventSupport();

    _init();
    
    return {
    	// TODO - allow this to be set as config option ?
    	BOOKMARK_FOLDER:			bookmarksService.toolbarFolder,
    	
    	// event types
    	EVENT_ADD_BOOKMARK: 		'EVENT_ADD_BOOKMARK',
    	EVENT_DEL_BOOKMARK:    		'EVENT_DEL_BOOKMARK',
    	EVENT_MOV_BOOKMARK:			'EVENT_MOV_BOOKMARK',
    	EVENT_LOAD_BOOKMARK: 		'EVENT_LOAD_BOOKMARK',
    	
    		
    	cleanup : function() {
    		bookmarksService.removeObserver(TabDockBookmarksListener);
    	},
    	
    	get bookmarkToolbarFolder() { return this._bookmarkFolder; },
    		
    	moveBookmarkGroupBefore : function(tarGroup, srcGroup) {
    		let movingDown = tarGroup ? tarGroup.ordinal > srcGroup.ordinal : true;
    		let targetIndex = tarGroup ? bookmarksService.getItemIndex(tarGroup.bookmarkIds[tarGroup.bookmarkIds.length - 1]) + 1 : 0;
    		let bookmarkIds = srcGroup.bookmarkIdArray;
    		
    		for(let i=0; i<bookmarkIds.length; i++) {
    			if(i != 0)
    				targetIndex = bookmarksService.getItemIndex(bookmarkIds[i-1]) + 1;
    			
    			bookmarksService.moveItem(bookmarkIds[i], this.BOOKMARK_FOLDER, targetIndex);
    		}

    	},
    	
      	bookmarkAdded: function(aItemId, aFolder, aIndex) {
      		try {
		    	_bookmarkFolder.containerOpen = true;
		    	let node = _bookmarkFolder.getChild(aIndex);
		  	  	
		  	  	/*
                 * Ignore about:blank, since in the process of adding a bookmark
                 * through the bookmark manager, it is created initially as
                 * blank (or at least does so on Linux)
                 */
		    	if(node.type == node.RESULT_TYPE_URI && node.uri !== "about:blank")
		    		_eventSupport.fire( { 'node' : node, }, this.EVENT_ADD_BOOKMARK);
		    	
	    	} finally {
		  	  	_bookmarkFolder.containerOpen = false;
			}
    	},
    	
    	bookmarkRemoved: function(aItemId, aFolder, aIndex) {
    		try {
		    	_bookmarkFolder.containerOpen = true;
		    	
		    	let node = _bookmarkFolder.getChild(aIndex);		  	  	
		    	if(node.type == node.RESULT_TYPE_URI)
		    		_eventSupport.fire( { 'node' : node, }, this.EVENT_DEL_BOOKMARK);
		    	
	    	} finally {
		  	  	_bookmarkFolder.containerOpen = false;
			}
    	},
    	
    	bookmarkMoved: function(aItemId, aFolder, aIndex) {
    		try {
        	   	_bookmarkFolder.containerOpen = true;
    			
        	   	let node = _bookmarkFolder.getChild(aIndex);
        	   	let nodePrevious = null;
    	  	  	
    	  	  	// Find first node prior to the moved node of the same type
    	  	  	for(let i=aIndex-1; i>=0; i--) {
    	  	  		if(_bookmarkFolder.getChild(i).type === node.type) {
    	  	 			nodePrevious = _bookmarkFolder.getChild(i);
    	  	  			break;
    	  	  		}	
    	  	  	}
    	  	
    	    	if(node.type == node.RESULT_TYPE_URI)
    	  	  		_eventSupport.fire( { 'node' : node, 'nodePrevious' : nodePrevious}, this.EVENT_MOV_BOOKMARK);
    	    	
    		} finally {
    	  	  	_bookmarkFolder.containerOpen = false;
    		}
    	},
    	
    	/**
         * Adds the specified uri as a bookmark to the bookmark folder, if it
         * does not already exist.
         * 
         * @param uri
         *            The uri to add a bookmark for.
         * @return The bookmark id of the added bookmark.
         */
    	addBookmark : function(uri) {
    	    return bookmarksService.insertBookmark(bookmarksService.toolbarFolder, uri, bookmarksService.DEFAULT_INDEX, "");
    	},
    	
    	/**
         * Loads the bookmarks from the specified bookmark folder by firing an
         * EVENT_LOAD_BOOKMARK event for each one.
         */
    	loadBookmarks: function() {
    		try {
	        	_bookmarkFolder.containerOpen = true;
	        	
	        	for (let i = 0; i < _bookmarkFolder.childCount; i ++) {
	        	    let node = _bookmarkFolder.getChild(i);
	        	  
                    // Only bookmarks supported so far
                    switch(node.type) {
                        case node.RESULT_TYPE_URI:
                            _eventSupport.fire( { 'node' : node, }, this.EVENT_LOAD_BOOKMARK);
                            break;
                    }
	        	}
    		}
        	finally {
            	_bookmarkFolder.containerOpen = false;
        	}
        },
        
        addListener: function(listener, type) { _eventSupport.addListener(listener, type); },
        removeListener: function(listener, type) { _eventSupport.removeListener(listener, type); },
    }
})();

