const EXPORTED_SYMBOLS = ['TabDockDragDrop'];

Components.utils.import('chrome://tabdock/content/preferences/preferences.js');
Components.utils.import('chrome://tabdock/content/tabdock.js');
Components.utils.import('chrome://tabdock/content/tabdock-utils.js');
Components.utils.import('chrome://tabdock/content/tabdock-bookmarks.js');
Components.utils.import('chrome://tabdock/content/tabdock-settings.js');

var TabDockDragDrop = function(aWindow) { 
	this.window = aWindow;
	this.document = aWindow.document;
}

/**
 * Provides drag/drop support for extension.
 * 
 */
TabDockDragDrop.prototype = {
        /** The current insert point, used when dropping */
		insertPoint: null,
		
		/**
         * Sets the insertion point for a subsequent drop event along with the
         * tab drop location ind. This code checks to see if the mouse is either
         * before or past the midway point of a given tab and positions the ind
         * before or after, respectively.
         */
		dragover : function(event) {
			let obj = event.target;
			let ind = this.document.getElementById('tabdock-movelocationindicator');
			
			// Only handle dropping tabgroups for now
			if(!event.dataTransfer.getData("text/fdock-groupId"))
				return;
			
			ind.collapsed = false;
			event.preventDefault();
			
			// There may be gaps where there isn't a xstabgroup. In that case,
			// do not modify the ind location.
			if (obj.nodeName != 'fdockTabgroup')
				return;
 
			// make sure to get ordinal sibling
			let ps = obj.boxObject.previousSibling;
			let locX = 0, locY = 0, xform = "";
			let rect = obj.getBoundingClientRect();

			if (TabDockSettings.orientation == 'horizontal') {
				if (event.clientX > (rect.left + rect.right) / 2) {
					locX = rect.left;
					TabDockDragDrop.insertPoint = obj;
				} else {
					// May not have a previous sibling, in which case need to
					// position accordingly
					locX = ps ? ps.getBoundingClientRect().left : rect.left
							- rect.width;
					TabDockDragDrop.insertPoint = ps;
				}
				
				locY = (TabDockSettings.location == 'top') ? rect.bottom : rect.top - (ind.clientHeight) - 5;
				locX += 2 * ind.clientWidth;
		
			} else {
				if (event.clientY > (rect.top + rect.bottom) / 2) {
					locY = rect.top;
					TabDockDragDrop.insertPoint = obj;
				} else {
					locY = ps ? ps.getBoundingClientRect().top : rect.top - rect.height;
					TabDockDragDrop.insertPoint = ps;
				}

				locX = (TabDockSettings.location == 'left') ? rect.right : rect.left  - (ind.clientWidth);
				locY += 1.5 * ind.clientHeight;
			}
			
			// rotate the drop icon as appropriate
			switch (TabDockSettings.location) {
				case 'top':
					xform = "matrix(1, 0, 0, 1,";
					break;
				case 'bottom':
					xform = "matrix(-1, 0, 0, -1,";
					break;
				case 'left':
					xform = "matrix(0, -1, 1, 0,";
					break;
				case 'right':
					xform = "matrix(0, 1, -1, 0,";	
					break;
			}

			ind.style.MozTransform = xform + locX + "px, " + locY + "px)";
		},


		dropTab : function(event) {
		    let ind = this.document.getElementById('tabdock-movelocationindicator');
			ind.collapsed = true;
			
			let srcElementGroupId = event.dataTransfer.getData('text/fdock-groupId');
			// TODO - Allow things other than tabgroups to be dropped?
			if(!srcElementGroupId)
				return;

			let movedGroup = this.window.TabDockService.findTabGroup(srcElementGroupId);
			let previousGroup = TabDockDragDrop.insertPoint;
			
			// Allow removal of bookmarks by dragging into tabs?
			if(movedGroup.isBookmark)
				return;
			
			// Re-order the tabgroups after moving
			movedGroup.ordinal = previousGroup ? previousGroup.ordinal - (-1): 0;
			TabDockUtils.reapplyOrdinals(this.document.getElementById("tabdock-tabs"));
		},

		dropBookmark : function(event) {
		    let ind = this.document.getElementById('tabdock-movelocationindicator');
			ind.collapsed = true;
			
			let srcElementGroupId = event.dataTransfer.getData('text/uri-list');
			    
			let movedGroup = this.window.TabDockService.findTabGroup(srcElementGroupId);
			let previousGroup = TabDockDragDrop.insertPoint;

			// TODO - Allow things other than tabgroups to be dropped?
			if(!movedGroup)
				return;
			
			if(!movedGroup.isBookmark) {
				movedGroup.bookmarkTabGroup();
				// After bookmarking, a new group should have been created
				movedGroup = this.window.TabDockService.findTabGroup(srcElementGroupId);
			}
			
			// Don't need to move tabgroups, since it will be taken care of
			// when the bookmarks are moved.
			TabDockBookmarks.moveBookmarkGroupBefore(previousGroup, movedGroup);
		},

		dragEnd: function(event) {
			var ind = this.document.getElementById('tabdock-movelocationindicator');
			ind.collapsed = true;
		},

		dragExit: function(event) {
			var ind = this.document.getElementById('tabdock-movelocationindicator');
			ind.collapsed = true;
		},
}

