const EXPORTED_SYMBOLS = ['TabDock'];

Components.utils.import("resource://gre/modules/Services.jsm");
Components.utils.import('resource://gre/modules/XPCOMUtils.jsm');

Components.utils.import('chrome://tabdock/content/tabdock-utils.js');
Components.utils.import('chrome://tabdock/content/preferences/preferences.js');
Components.utils.import('chrome://tabdock/content/tabdock-settings.js');
Components.utils.import('chrome://tabdock/content/tabevents.js');
Components.utils.import('chrome://tabdock/content/tabdock-bookmarks.js');
Components.utils.import('chrome://tabdock/content/tabdock-dragdrop.js');
Components.utils.import('chrome://tabdock/content/tabgroup/tabgroup.js');
Components.utils.import('chrome://tabdock/content/tabpreviews/tabpreviews.js');

var ioService = Services.io;

var TabDock = function(aWindow) {
	
	TabDockUtils.dump("start init tabdock\n");
	
	this.window = aWindow;
	this.document = aWindow.document;
	this.gBrowser = aWindow.gBrowser;
	
	this.tabdockOverlay = aWindow.document.createElement('fdockOverlay');
    this.window.document.getElementById('appcontent').appendChild(this.tabdockOverlay);
	
	this.dragdrop = new TabDockDragDrop(aWindow);
	this.tabpreviews = new TabPreviews(aWindow);
	this.tabevents = new TabEvents(aWindow);
	
	/** The firefox menu, if applicable */
	this._firefoxMenu = null;
	
	this._tabStringBundle = this.document.getElementById("tab-strings");
	
	aWindow.TabDockService = this;
	aWindow.TabDockService.setup();

	TabDockUtils.dump("done init tabdock\n");	
}
    
TabDock.prototype = {
	
	/** Maps tabgroup ids to tab groups */
	tabgroupIdMap: {},
    
	/**
     * Moves the status panel so that it is appropriately positioned when
     * appcontent is a stack.
     */
	moveStatusPanel: function() {
	    TabDockUtils.dump("Moving statusbar\n");
	    
	    let statusPanel = this.document.getElementById('statusbar-display');
	    let appcontent = this.document.getElementById('appcontent');
	      
	    // wraps the status panel for pushing down
	    let statusbarbox = this.document.createElement('vbox');
	    statusbarbox.setAttribute('id', 'statusbarbox');
	    
	    // used to push the status panel down to the bottom
	    let statusbarspacer = this.document.createElement('vbox');
	    statusbarspacer.setAttribute('flex', '1');
	    
	    appcontent.appendChild(statusbarbox);
	    statusbarbox.appendChild(statusbarspacer);
	    statusbarbox.appendChild(statusPanel);
	},
	
    /**
     * Moves the status panel so that it is appropriately positioned when
     * appcontent is not a stack (when addon is disabled).
     */
	unmoveStatusPanel: function() {
	    let statusbarbox = this.document.getElementById('statusbarbox');
	    let statusPanel = this.document.getElementById('statusbar-display');
        let appcontent = this.document.getElementById('appcontent');
        
        appcontent.appendChild(statusPanel);
        appcontent.removeChild(statusbarbox);
	},
	
	
    /**
     * Creates a tabgroup, adding it to the tab dock. If creating as a bookmark,
     * be sure to call add bookmark afterwards, since createAsBookmark only
     * selects the container the tab group goes into.
     * 
     * @param groupId
     *            The groupId for the newly created group.
     * @param createAsBookmark
     *            Whether to put the tab group in the bookmark container or the
     *            tab container.
     * 
     * @return The newly created tabgroup.
     */
	createTabGroup: function(groupId, createAsBookmark) {
    	let container = createAsBookmark ? this._bookmarksContainer : this._tabsContainer;
    	
    	let tabGroup = this.document.createElement('fdockTabgroup');
    	tabGroup.groupId = groupId;
    	
    	this.tabgroupIdMap[groupId] = tabGroup;
    	
    	// Apply ordinal values 2 apart so that we can reorder the groups easily
		// later when moving groups.
    	let lastOrdinal = container.lastChild ? container.boxObject.lastChild.ordinal  : 0;
    	tabGroup.ordinal = (lastOrdinal - 0) + 2;
    	
    	return container.appendChild(tabGroup);
    },
    
	 
	/**
     * Gets the tab's groupId. Currently, this is defined as the host, with a
     * fallback to the full URI if the tab URI is not a URL with a hostname.
     * This applies to about:config, file paths and anything else that doesn't
     * have a host.
     * 
     * The groupId may be based in the future on the favIcon (if it exists) in
     * order to allow subdomains, such as yro.slashdot.org and
     * linux.slashdot.org to group together. This also solves the case of
     * google.com/calendar being grouped with www.google.com (search).
     * 
     * @param tab
     *            The tab to get the groupId for.
     * @return The groupId for the tab.
     */
    getTabGroupId: function(tab) {
        let currentUri = this.gBrowser.getBrowserForTab(tab).currentURI;
        
        try {
            return currentUri.host || currentUri.asciiSpec;
        } catch(err) {
            try {
                return currentUri.asciiSpec;
            } catch(err) {
                return tab.label;
            }
        }
    },

    
    /**
     * Gets the groupId from a uriString. This should be kept in sync with
     * getTabGroupId.
     * 
     * @param uriString
     *            A string representing a URI to get the host of.
     * @return The host, if applicable, the uriString otherwise.
     */
    getTabGroupIdFromUriString: function(uriString) {
    	try {
    	    return ioService.newURI(uriString, null, null).host || uriString;
    	} catch(err) {
    	    return uriString;
    	}
    },
    
    
    /**
     * Finds the tabgroup corresponding to a groupId.
     * 
     * @param groupId
     *            The group to look for.
     * @return The tabgroup if found, undefined otherwise.
     */
    findTabGroup: function(groupId) {
        if(this.tabgroupIdMap.hasOwnProperty(groupId))
            return this.tabgroupIdMap[groupId];
        else
            return undefined;
    },
    
    
    /**
     * Gets a tabgroup, creating it if necessary.
     * 
     * @param groupId
     *            The groupId to find / create the tabgroup for.
     * @param createAsBookmark
     *            Whether to create the group in the bookmark container if no
     *            group was found.
     * @return The tabgroup for the given groupId.
     */
    getTabGroup: function(groupId, createAsBookmark) {
    	return this.findTabGroup(groupId) || this.createTabGroup(groupId, createAsBookmark);
    },
    
    
    /**
     * Adds a tab to the tabdock, causing a group to be created if necessary.
     * 
     * @param tab
     *            The tab to add.
     */
    addTab: function(tab) {
    	try {
    	    let groupId = this.getTabGroupId(tab);
            
        	// If the page is going to about:blank, it may just be loading
  			// In that case, the label might be the URI that is loading
    	    let tabStringBundle = this.document.getElementById("tab-strings");
        	let newTabString = tabStringBundle.getString("tabs.emptyTabTitle");
      		if(groupId === "about:blank" && tab.label != newTabString && tab.label != "") {
      		    groupId = this.getTabGroupIdFromUriString(tab.label);
      		}
      		
      		TabDockUtils.dump("adding tab to group " + groupId + "\n");
      		let tabGroup = this.getTabGroup(groupId, false);
    			
			tabGroup.addTab(tab);
    	} catch (err) {
    		TabDockUtils.dump("error adding tab: " + err + "\n");
            TabDockUtils.dump(err.stack);
    	}
    },

    
    /**
     * Moves tabs between bookmarks and tabs by creating a new group and moving
     * the tabs from the old group to the new group.
     * 
     * @param oldTabGroup
     *            The tabgroup to move tabs from.
     * @param toBookmarks
     *            Whether moving to or from bookmarks
     * @return newTabGroup A newly created group with the tabs of the old group.
     */
    moveTabGroup: function(oldTabGroup, toBookmarks) {
       
        // save off the tabs and group id for the old group
        let groupId = oldTabGroup.groupId;
        let tabs = new Array();
        while(oldTabGroup.tabCount > 0)
            tabs.push(oldTabGroup.removeTab(oldTabGroup.firstTab));
        
        // make sure old group is removed before creating the new one
        if(oldTabGroup && oldTabGroup.killGroup)
            oldTabGroup.killGroup();          
        
        // now add new group with all of the tabs
        let newTabGroup = this.window.TabDockService.createTabGroup(groupId, toBookmarks);
    	while(tabs.length > 0)
    		newTabGroup.addTab(tabs.pop());
    	
    	return newTabGroup;
    },

    
	/**
     * Sets up the tab dock, loading the bookmarks and tabs as well as setting
     * the orientation and popup direction for the various elements. This
     * function is called whenever the preference for the layout direction is
     * updated. It also needs to be called on startup in order to get the
     * correct orientation and load bookmarks / initial tabs.
     * 
     * @param horizontal
     *            Sets the dock orientation to horizontal if true, vertical
     *            otherwise.
     */
	setupTabdock: function() {
		let mode = CurrentPrefs['mode'];
		let location = (mode == 'integrated') ? 'top' : CurrentPrefs['location'];

	    TabDockSettings.location = location;
		TabDockSettings.orientation = TabDockSettings.orientOfLocation[location];
		TabDockSettings.popupDirection = TabDockSettings.popupDirectionOfLocation[location];
		
		let tabdock = this.document.getElementById('tabdock');
		let insertParent = this.document.getElementById((TabDockSettings.insertParentOfLocation[mode])[location]);
		let insertAnchor = this.document.getElementById((TabDockSettings.insertAnchorOfLocation[mode])[location]);
	 	
		insertParent.insertBefore(tabdock, insertAnchor);
		
	 	// Set the orient, which should cause the children to orient due to css
		tabdock.setAttribute('location', location);
	    tabdock.setAttribute('iconSize', TabDockSettings.iconsize);
		
		let tabdockToolbar = this.document.getElementById('tabdock-toolbar');
		tabdockToolbar.setAttribute('orient', TabDockSettings.orientation);
		tabdockToolbar.setAttribute('location', location);
	    
	    let tabdockOverlay = this.document.getElementById('tabdock-overlay');
	    tabdockOverlay.setAttribute('orient', TabDockSettings.orientation);
	    tabdockOverlay.setAttribute('location', location);
	    
		// Even with css to make arrowscrollbox orient correct, the scroll
		// buttons only work if we manually set the orient
	    let tabdockContainer = this.document.getElementById('tabdock-container');
		tabdockContainer.setAttribute('orient', TabDockSettings.orientation);
		
		this._bookmarksContainer = this.document.getElementById("tabdock-bookmarks");
		this._tabsContainer = this.document.getElementById("tabdock-tabs");
		
		this.setupAppMenu();
		
		this.setMode(mode);
        this.setAlignment(CurrentPrefs['alignment']);
        this.showTabsToolbar(CurrentPrefs['firefoxTabs']);
        this.showTabdockExtras(CurrentPrefs['tabdockExtras']);
        this.setAutohide(CurrentPrefs['dock.autohide']);
        this.setDockStyle(CurrentPrefs['dock.style']);
        this.setDockAppearance(CurrentPrefs['dock.appearance']);
        this.setGreyoutInactiveIcons(CurrentPrefs['style.greyoutInactiveIcons']);
		
	    this.reloadTabDockState();
	},
	
	
	/**
     * Loads the state for the tabdock, creating all groups, bookmarks, existing
     * tabs, etc.
     */
	reloadTabDockState: function() {
	 	// Remove all old state
	 	TabDockUtils.removeAllChildren(this.document.getElementById('tabdock-bookmarks'));
	 	TabDockUtils.removeAllChildren(this.document.getElementById('tabdock-tabs'));
	 	this.tabgroupIdMap = {};     // clear out tabgroup mappings
		
	 	// Make sure the Firefox menu nodes position correctly
	 	if(this._firefoxMenu)
	 		for(let i = 0; i < this._firefoxMenu.childNodes.length; i++)
	 			this._firefoxMenu.childNodes[i].setAttribute('position', TabDockSettings.popupDirection);
		
	 	// Load bookmarks and tabs in order to restore state
		this.loadBookmarks();
		this.loadTabs();
	},
	

	/**
     * Goes through all the tabs and puts them into tabgroups. This is used on
     * startup (to load tabs opened before the addon started) and when changing
     * the orientation, since the DOM node for the tabdock container needs to be
     * moved and the state is lost.
     */
	loadTabs: function() {
	    let container = this.gBrowser.tabContainer;
    	for(let i = 0; i < container.itemCount; i++)
    		this.addTab(container.getItemAtIndex(i));
    },
    
    
    /**
     * Loads the bookmarks for when the tabdock is being created. Only
     * difference in the event is that EVENT_LOAD_BOOKMARK is fired from
     * TabDockBookmarks.loadBookmarks(). This is to fix a bug with bookmarks
     * being added again to every tabgroup in an existing window when a new
     * window is opened.
     */
	loadBookmarks: function() {
		TabDockBookmarks.addListener(this, TabDockBookmarks.EVENT_LOAD_BOOKMARK);
		TabDockBookmarks.loadBookmarks();
		TabDockBookmarks.removeListener(this, TabDockBookmarks.EVENT_LOAD_BOOKMARK);
    },
	
    
	/**
     * Adds an icon to the tab dock for the firefox menu, by cloning the node.
     * This node may not exist on some platforms (e.g. Windows), where the menu
     * becomes a part of the titlebar, so we need to check here, and everywhere
     * we use it.
     */
	setupAppMenu: function() {
		let originalFirefoxMenu = this.document.getElementById('appmenu-toolbar-button')
		let separator = this.document.getElementById('tabdock-menu-separator');
		let dockMenu = this.document.getElementById('tabdock-menus');
		let contextmenuOption = this.document.getElementById('tabdock-contextmenu-firefoxMenu');
		
		if(originalFirefoxMenu) {
			this._firefoxMenu = originalFirefoxMenu.cloneNode(true);
			this._firefoxMenu.setAttribute('id', 'fdock-firefox-menu');
			
			dockMenu.insertBefore(this._firefoxMenu, separator);
	        this.showFirefoxMenu(CurrentPrefs['firefoxMenu']);
		} else {
			contextmenuOption.setAttribute('collapsed', 'true');
	        this.showFirefoxMenu(false);
		}
	},
	
    setDockmode: function(enabled) {
         this.document.getElementById('tabdock').setAttribute('dockmode', enabled);
         this.document.getElementById('tabdock-overlay').setAttribute('dockmode', enabled);
    },
    
    setMode: function(mode) {
         this.document.getElementById('TabsToolbar').setAttribute('fdock-mode', mode);
         this.document.getElementById('tabdock').setAttribute('mode', mode);
         this.document.getElementById('tabdock-overlay').setAttribute('mode', mode);
    },
    
    setGreyoutInactiveIcons: function(enabled) {
		this.document.getElementById('tabdock').setAttribute('greyoutInactiveIcons', enabled);
    },
	
    isAutoHidden: function() {
		return this.document.getElementById('tabdock-overlay').getAttribute('autohideDock');
    },
	
    setAutohide: function(autohide) {
		this.document.getElementById('tabdock-overlay').setAttribute('autohideDock', autohide);
    },
    
    setDockStyle: function(style) {
		this.document.getElementById('tabdock-overlay').setAttribute('style', style);
    },
    
    setDockAppearance: function(appearance) {
        this.document.getElementById('tabdock-overlay').setAttribute('appearance', appearance);
    },
    
    setTitlechanged: function(titlechanged) {
		this.document.getElementById('tabdock-overlay').setAttribute('titlechanged', titlechanged);
    },
    
	showTabsToolbar: function(show) {
		show |= CurrentPrefs['mode'] == "integrated";
		this.document.getElementById('TabsToolbar').setAttribute('TabsToolbarCollapse', !show);
  	},
  	
    showFirefoxMenu: function(show) {
        this.document.getElementById('tabdock-menus').setAttribute('collapsed', !show);
    },
    
    showTabdockExtras: function(show) {
		this.document.getElementById('tabdock-extras').setAttribute('collapsed', !show);
    },
    
  	/**
     * Set the alignment for the tabdock.
     * 
     * @param alignment
     *            The alignment to set, one of: start, center, end. Anyother
     *            value causes alignment of center.
     */
  	setAlignment: function(alignment) {
  		TabDockSettings.alignment = alignment;
  		
  		let tabdockToolbar = this.document.getElementById('tabdock-toolbar');
  		tabdockToolbar.setAttribute('alignment', alignment);
  	},

  	
  	// -------------------------------------------------------------------------
  	// HANDLER FUNCTIONS
  	
	/**
     * Handles events for when preferences change, applying any necessary
     * changes to the add-on.
     */
  	prefChanged: function(name, value) {  
	
  	  TabDockUtils.dump("pref change: " + name + " -> " + value + "\n");
  	    
	  switch(name)
	  { 
	  	case "mode":
	    case "location":
	        return this.setupTabdock();
	    case "dock.enabled":
	        return this.setupTabdock();
	    case "dock.autohide":
	        return this.setAutohide(value);
	    case "dock.style":
	        return this.setDockStyle(value);
        case "dock.appearance":
            return this.setDockAppearance(value);
        case "style.greyoutInactiveIcons":
            return this.setGreyoutInactiveIcons(value);
	    case "alignment":
	    	return this.setAlignment(value);
	    case "firefoxTabs":
	    	return this.showTabsToolbar(value);
        case "firefoxMenu":
	        return this.showFirefoxMenu(value);
        case "tabdockExtras":
            return this.showTabdockExtras(value);
        case "debug":
	    	return TabDockUtils.enableDebug(value);
	  }
	},

	
  	/**
     * Handles events for the add-on. Usually, I would prefer to register
     * separate functions, but we need a reference to the TabDock object. I may
     * be missing something here, but this seems to work well enough.
     * 
     * @param aEvent
     *            The event (either DOM or internal) to handle
     */
	handleEvent : function(aEvent) {

		switch (aEvent.type)
		{
			case TabDockBookmarks.EVENT_ADD_BOOKMARK:
			case TabDockBookmarks.EVENT_LOAD_BOOKMARK:
				return this.onBookmarkAdded(aEvent);
			case TabDockBookmarks.EVENT_MOV_BOOKMARK:
				return this.onBookmarkMoved(aEvent);
			case TabDockBookmarks.EVENT_DEL_BOOKMARK:
				return this.onBookmarkRemoved(aEvent);
			case TabEvents.EVENT_TAB_OPENED:
				return this.onTabOpen(aEvent);
            case TabEvents.EVENT_TAB_SELECTED:
				return this.onTabSelect(aEvent);
            case TabEvents.EVENT_TAB_CLOSED:
				return this.onTabClose(aEvent);
			case TabEvents.EVENT_TAB_ATTR_CHANGED:
				return this.onTabAttrChange(aEvent);
            case TabEvents.EVENT_TAB_TITLE_CHANGED:
                return this.onTabTitleChange(aEvent);
            case TabEvents.EVENT_TAB_TITLE_CHANGED_CLEARED:
                return this.onTabTitleChangeCleared(aEvent);
            case TabEvents.EVENT_TAB_UNREAD:
                return this.onTabUnread(aEvent);
            case TabEvents.EVENT_TAB_UNREAD_CLEARED:
                return this.onTabUnreadCleared(aEvent);
            case TabEvents.EVENT_TAB_BUSY:
                return this.onTabBusy(aEvent);
            case TabEvents.EVENT_TAB_BUSY_CLEARED:
                return this.onTabBusyCleared(aEvent);
			case Preferences.EVENT_PREFERENCE_CHANGED:
			    return this.prefChanged(aEvent.name, aEvent.value);
		}
	},
  	
  	
    // ------------------------------------------------------------------------
    // LISTENER FUNCTIONS
  	
	onBookmarkAdded: function(event) {
	    let node = event.node;
	    let groupId = this.window.TabDockService.getTabGroupIdFromUriString(node.uri);
	    let tabGroup = this.window.TabDockService.findTabGroup(groupId);
		
		// We already have tabs in a tabgroup and they need to be moved over to
		// the bookmark section
	    if(tabGroup && !tabGroup.isBookmark)
	    	tabGroup = this.window.TabDockService.moveTabGroup(tabGroup, true);
	    // There is no tabgroup, so we need to create a bookmark one
	    else if(!tabGroup)
	    	tabGroup = this.window.TabDockService.createTabGroup(groupId, true);
	    
		tabGroup.addBookmark(node.itemId, node.uri, node.icon);
	},
	
	onBookmarkRemoved: function(event) {
	    let node = event.node;
	    let groupId = this.window.TabDockService.getTabGroupIdFromUriString(node.uri);
	    let tabGroup = this.window.TabDockService.findTabGroup(groupId);
		
		tabGroup.removeBookmark(node.itemId, node.uri);
		
		if(tabGroup.bookmarkCount == 0) {
			if(tabGroup.tabCount > 0)
				this.window.TabDockService.moveTabGroup(tabGroup, false);
			
			// Kill (remove) the group since it should not have any tabs or
			// bookmarks at this point
			if(tabGroup && tabGroup.killGroup)
			    tabGroup.killGroup();
		}
	},
	
	onBookmarkMoved: function(event) {
	    let node = event.node;
	    let groupId = this.window.TabDockService.getTabGroupIdFromUriString(node.uri);
	    let group = this.window.TabDockService.findTabGroup(groupId);
		
		var prevGroup = null;
		var nodePrevious = event.nodePrevious;
		if(nodePrevious != null) {
			var prevGroupId = this.window.TabDockService.getTabGroupIdFromUriString(nodePrevious.uri);
			prevGroup = this.window.TabDockService.findTabGroup(prevGroupId);
		}

		group.ordinal = prevGroup ? prevGroup.ordinal - (-1): 0;
		TabDockUtils.reapplyOrdinals(this.document.getElementById("tabdock-bookmarks"));
	},
	
	
	onTabOpen: function(event) {
	    let tab = event.tab;
	    
		this.addTab(tab);
		
	    let tabGroup = TabGroup.getTabGroupForTab(tab);
        this.window.TabDockService.tabpreviews.addTab(tab, tabGroup);
    },
    
	onTabSelect: function(event) {
	    TabGroup.selectTab(event.tab);
    },
    
	onTabClose: function(event) {
		let tab = event.tab;
	 	let tabGroup = TabGroup.getTabGroupForTab(tab);
	
		this.window.TabDockService.tabpreviews.removeTab(tab, tabGroup);
	    TabGroup.removeTab(event.tab);
    },
    
    onTabAttrChange: function(event) {
		let tab = event.tab;
        let tabGroup = TabGroup.getTabGroupForTab(tab);
        
		if(tabGroup) {
		    tabGroup.updateTab(tab);
		    
			/*
             * Do not move tabs in the connecting state, since they will be sent
             * to the same tabgroup as about:blank. They should be fine where
             * they currently are until they have finished loading and can be
             * moved to a new group if necessary.
             */
	        let tabStringBundle = this.document.getElementById("tab-strings");
	        let connectingString = tabStringBundle.getString("tabs.connecting");
			if( tab.label != connectingString && this.getTabGroupId(tab) != tabGroup.groupId )
			    this.addTab(tabGroup.removeTab(tab));
		}
		
    },
    
    onTabTitleChange: function(event) {
        let tab = event.tab;
        
        this.setTitlechanged(true);
        TabGroup.setTitleChange(tab);
        
        this.window.TabDockService.tabpreviews.setTitleChanged(tab, TabGroup.getTabGroupForTab(tab));
    },
    
    onTabTitleChangeCleared: function(event) {
        this.setTitlechanged(this.tabevents.titleChangedTabsCount != 0);
        TabGroup.clearTitleChange(event.tab);
    },
    
    onTabUnread: function(event) {
        TabGroup.setUnread(event.tab);
    },
    
    onTabUnreadCleared: function(event) {
        TabGroup.clearUnread(event.tab);
    },
    
    onTabBusy: function(event) {
        TabGroup.setBusy(event.tab);
    },
    
    onTabBusyCleared: function(event) {
        let tab = event.tab;
        let tabGroup = TabGroup.getTabGroupForTab(tab);
        
        TabGroup.clearBusy(tab);
        
        this.window.TabDockService.tabpreviews.updatePreview(tab, tabGroup);
    },
    
    // ------------------------------------------------------------------------
	// SETUP AND TEARDOWN
    
	/**
     * Causes the preferences to be loaded for the add-on, causing
     * initialization where appropriate.
     */
	loadPreferences: function() {
	    
	    this.setupTabdock();
		this.prefChanged("debug", CurrentPrefs['debug']);

	}, 
	
	setup: function() {
	    let container = this.gBrowser.tabContainer;

		this.tabevents.addListener(this);
		
		Preferences.addListener(this, Preferences.EVENT_PREFERENCE_CHANGED);
		TabDockBookmarks.addListener(this, TabDockBookmarks.EVENT_ADD_BOOKMARK);
		TabDockBookmarks.addListener(this, TabDockBookmarks.EVENT_MOV_BOOKMARK);
		TabDockBookmarks.addListener(this, TabDockBookmarks.EVENT_DEL_BOOKMARK);
		
	    this.moveStatusPanel();
		this.loadPreferences();
	},
	
	cleanup: function() {
	    let container = this.gBrowser.tabContainer;
		
        this.tabevents.removeListener(this);
		
	    Preferences.removeListener(this, Preferences.EVENT_PREFERENCE_CHANGED);
		TabDockBookmarks.removeListener(this, TabDockBookmarks.EVENT_ADD_BOOKMARK);
		TabDockBookmarks.removeListener(this, TabDockBookmarks.EVENT_MOV_BOOKMARK);
		TabDockBookmarks.removeListener(this, TabDockBookmarks.EVENT_DEL_BOOKMARK);

		this.unmoveStatusPanel();
	    this.window.document.getElementById('appcontent').removeChild(this.tabdockOverlay);
	    
	    this.window.TabEvents.cleanup();
		delete this.window.TabDockService;
	},
}
