const EXPORTED_SYMBOLS = ['TabGroup'];

Components.utils.import('chrome://tabdock/content/tabdock-bookmarks.js');
Components.utils.import('chrome://tabdock/content/tabdock-utils.js');
Components.utils.import('chrome://tabdock/content/tabdock-settings.js');
Components.utils.import('chrome://tabdock/content/preferences/preferences.js');

function TabGroup(obj, window) {
	this.obj = obj;
	this.window = window;
	this.gBrowser = window.gBrowser;

	obj.uri = "";
	obj.image = null;
	obj.isBookmark = false;
	obj.bookmarkIdArray = Array();
	obj.bookmarkUriArray = Array();
	obj.setAttribute('isBookmark', 'false');
	
	obj.unreadCounter = new SetAttributeCounter(0, 0);
	obj.unreadCounter.addThreshold(0, obj, 'unread', 'false');
	obj.unreadCounter.addThreshold(1, obj, 'unread', 'true');
	
	obj.titlechangedCounter = new SetAttributeCounter(0, 0);
	obj.titlechangedCounter.addThreshold(0, obj, 'titlechanged', 'false');
	obj.titlechangedCounter.addThreshold(1, obj, 'titlechanged', 'true');

	obj.busyCounter = new SetAttributeCounter(0, 0);
	obj.busyCounter.addThreshold(0, obj, 'busy', 'false');
	obj.busyCounter.addThreshold(1, obj, 'busy', 'true');

	obj.tabCounter = new SetAttributeCounter(0, 0);
	obj.tabCounter.addThreshold(0, obj, 'state', 'closed');
	obj.tabCounter.addThreshold(1, obj, 'state', 'opened');
	obj.tabCounter.addThreshold(2, obj, 'state', 'group');
	
	let closeTabMenuItem = window.document.getAnonymousElementByAttribute(obj, "anonid", "tabgroup-menu-closeTabMenuItem");
	let refreshTabMenuItem = window.document.getAnonymousElementByAttribute(obj, "anonid", "tabgroup-menu-refreshTabMenuItem");
	
	obj.tabCounter.addThreshold(0, refreshTabMenuItem, 'disabled', 'true');
	obj.tabCounter.addThreshold(1, refreshTabMenuItem, 'disabled', 'false');
	obj.tabCounter.addThreshold(0, closeTabMenuItem, 'disabled', 'true');
	obj.tabCounter.addThreshold(1, closeTabMenuItem, 'disabled', 'false');
	
	/**
     * Starts a drag event, setting the data and drag image.
     */
	this.dragstart = function(event) {
		var dt = event.dataTransfer;
		dt.setData("text/uri-list", obj.groupId);
		dt.setData("text/plain", obj.groupId);
		dt.setData("text/fdock-groupId", obj.groupId);
		
		dt.addElement(obj);
		
		if(obj.firstTab) {
			let canvas = TabDockUtils.generatePreview(obj.firstTab);
			
			switch(TabDockSettings.location) {
				case 'bottom':
					dt.setDragImage(canvas, 0, canvas.height);
					break;
				case 'right':
					dt.setDragImage(canvas, canvas.width, 0);
					break;
				default:
					dt.setDragImage(canvas, 0, 0);
					break;
			}
	
		}
		
		this.window.document.getElementById("tabgroup-tooltip").setAttribute('hidden', 'true');
	};
	
	this.dragend = function(event) {
	    this.window.document.getElementById("tabgroup-tooltip").removeAttribute('hidden');
	    
		if(obj.hovering)
			obj.showTooltip();
	};
	
    this.setImage = function(image, isBookmarkImage) {
        if( (obj.isBookmark && isBookmarkImage) || !obj.isBookmark )
            obj.image = image;
    };
    
	/**
     * Adds a bookmark to the list of bookmarks. Sets the isBookmark attribute
     * and creates the bookmark menu item. This does not move the group from the
     * tabs section to the bookmarks section.
     */
	this.addBookmark = function(bookmarkId, bookmarkUri, bookmarkImage) {
		obj.bookmarkIdArray.push(bookmarkId);
		obj.bookmarkUriArray.push(bookmarkUri);
		obj.uri = bookmarkUri;
		obj.isBookmark = true;
		obj.setAttribute('isBookmark', 'true');

		if(bookmarkImage)
		    this.setImage(bookmarkImage, true);
		
		let menuitem = this.window.document.createElement('menuitem');
		menuitem.setAttribute('label', bookmarkUri);
		menuitem.addEventListener('command', function() { this.newTab(bookmarkUri); }.bind(this), false);
		menuitem.bookmarkUri = bookmarkUri;
		menuitem.bookmarkId = bookmarkId;
		menuitem.tabGroup = obj;
		
		let bookmarksMenuPopup =  this.window.document.getAnonymousElementByAttribute(obj, "anonid", "bookmarksMenuPopup") ;
		bookmarksMenuPopup.appendChild(menuitem);
 		
 		let childNodes = obj.leftClickPopupMenu.childNodes;
 		for(let i=0; i<childNodes.length; i++)
			childNodes[i].tab.setAttribute('fdock-isbookmark', 'true');
	};
	
	/**
     * Removes a bookmark from the tab group. This simply removes the bookmark
     * from the list. The logic to move the group to the tabs section, if the
     * last bookmark is deleted, is elsewhere.
     * 
     * @param bookmarkId
     *            The bookmark id to remove
     * @param bookmarkUri
     *            The uri of the corresponding bookmark
     */
	this.removeBookmark = function(bookmarkId, bookmarkUri) {
		TabDockUtils.removeFromArray(obj.bookmarkIdArray, bookmarkId);
		TabDockUtils.removeFromArray(obj.bookmarkUriArray, bookmarkUri);
		
		var childNodes = obj.bookmarksMenuPopup.childNodes;
		for(let i=0; i<childNodes.length; i++)
			if(childNodes[i].bookmarkId === bookmarkId) {
				obj.bookmarksMenuPopup.removeChild(childNodes[i]);
				break;
			}
	};
	
	/**
     * Reloads all the tabs in the group.
     */
	this.reloadTabGroup = function(event) {
	    let childNodes = obj.leftClickPopupMenu.childNodes;
		for(let i=0; i<childNodes.length; i++)
			this.gBrowser.reloadTab(childNodes[i].tab);
	};

	/**
     * Bookmarks all the tabs in the tabgroup.
     */
	this.bookmarkTabGroup = function(event) {
	    let childNodes = obj.leftClickPopupMenu.childNodes;
		
		/*
         * If this isn't a bookmark group, bookmarking a tab will cause the
         * current group to get killed in the process, so we need to make sure
         * we set aside all the tabs so we can bookmark them.
         */
		var tabs = Array();
		for(let i=0; i<childNodes.length; i++)
			tabs.push(childNodes[i].tab);
		
		for(let i=0; i<tabs.length; i++)
			TabDockBookmarks.addBookmark(this.gBrowser.getBrowserForTab(tabs[i]).currentURI);
	};
	
	this.removeBookmarks = function(event) {
		// TODO - implement
	};

	/**
     * Closes a tabgroup by removing all the tabs from it. If the tabgroup is a
     * bookmark, then all the open tabs are closed, but the bookmark still
     * remains.
     */
	this.closeTabGroup = function(event) {
	    let childNodes = obj.leftClickPopupMenu.childNodes;
		
		// Don't use while loop, gets stuck if tab is last in the browser
	    let childNodeCount = childNodes.length;			
		for (let i=0; i<childNodeCount; i++)
			this.gBrowser.removeTab(obj.leftClickPopupMenu.firstChild.tab);
	};

	/**
     * Creates a new tab and selects it. If no argument is given, then the first
     * bookmark is opened instead.
     * 
     * @param bookmarkUri
     *            The bookmark uri to open.
     */
	this.newTab = function(bookmarkUri) {
	    let uri = bookmarkUri || obj.bookmarkUriArray[0];
	    
		this.gBrowser.selectedTab = this.gBrowser.addTab(uri);
	};
	
	this.updateTab = function(tab) {
  	    this.setImage(tab.image, false);
  	    
  	  TabGroup.tabPropertiesMap.get(tab).menuitem.setAttribute('label', tab.label);
	};
	
	/**
     * Updates the state of the tab/tabgroup for when a tab is selected. This
     * sets the selected attribute and removes the unread, titlechanged
     * attributes.
     * 
     * @param tab
     *            The tab that has been selected.
     * @return The selected tab.
     */
	this.selectTab = function(aTab) {
	    try {
	        
    	    let tabProperties = TabGroup.tabPropertiesMap.get(aTab);
  
    		obj.setAttribute('selected', 'true');
    		
    		// Let tabpreviews know that the tab has been selected
            this.window.TabDockService.tabpreviews.setSelected(aTab, obj);
            
    		if(TabGroup.lastSelectedTabGroup && TabGroup.lastSelectedTabGroup != obj) {
    		    TabGroup.lastSelectedTabGroup.setAttribute('selected', 'false');
    		}
    		
    		if(TabGroup.lastSelectedTab) {
                let previousTabProperties = TabGroup.tabPropertiesMap.get(TabGroup.lastSelectedTab);
                previousTabProperties.menuitem.setAttribute('selected', false);
                previousTabProperties.menuitem.setAttribute('checked', false);
    		}
    		
    	    TabGroup.lastSelectedTab = aTab;
    		TabGroup.lastSelectedTabGroup = obj;
    			
    		tabProperties.read = true;
    			
            tabProperties.menuitem.setAttribute('selected', true);
            tabProperties.menuitem.setAttribute('checked', true);
            
    		return aTab;
	    } catch(err) {
	        TabDockUtils.dump("error selecting tab: " + err+ "\n");
	        TabDockUtils.dump(err.stack);
	    }
	};

	/**
     * Adds a tab to the tabgroup, creating the left click menu item. If the tab
     * has not been read yet and it is being opened in the background (i.e. not
     * selected), the unread attribute is set.
     * 
     * @param tab
     *            The tab to add.
     * @return The added tab.
     */
	this.addTab = function(tab) {
	    
	    let tabProperties = TabGroup.tabPropertiesMap.get(tab) || {};
	
	    TabGroup.tabPropertiesMap.set(tab, tabProperties);
	    
	    let menuitem = this.window.document.createElement('fdockTabgroupItem');
	  	menuitem.tab = tab;
	    menuitem.setAttribute('label', tab.label);
	  	
	    tabProperties.menuitem = menuitem;
	    tabProperties.tabGroup = obj;
	    tabProperties.oldLabel = tab.label;
        
	    this.setImage(tab.image);
		obj.leftClickPopupMenu.appendChild(menuitem);
		obj.tabCounter.increment();

		if(CurrentPrefs['manageTabPinning'] == true)
    		if(obj.isBookmark)
    			this.gBrowser.pinTab(tab);
    		else
    			this.gBrowser.unpinTab(tab);
        
		if(tab.selected)
			obj.selectTab(tab);
		
		if(tabProperties.titlechanged)
		    TabGroup.setTitleChange(tab);
		if(tabProperties.unread)
	        TabGroup.setUnread(tab);
		    
		if(obj.isBookmark)
			tab.setAttribute('fdock-isbookmark', 'true');
		else
			tab.removeAttribute('fdock-isbookmark');
		    
		return tab;
	};

	/**
     * Removes a tab from the tabgroup, getting rid of the group if necessary.
     * 
     * @param tab
     *            The tab to remove.
     * @return The removed tab.
     */
	this.removeTab = function(tab) {
	    let tabProperties = TabGroup.tabPropertiesMap.get(tab);
		
		obj.leftClickPopupMenu.removeChild(tabProperties.menuitem);
		obj.tabCounter.decrement();
		
		TabGroup.clearTitleChange(tab);
		TabGroup.clearUnread(tab);
		TabGroup.clearBusy(tab);

		if(obj.tabCount > 0)
		    this.setImage(obj.firstTab.image, false);
		// remove tabgroup if no more nodes and not bookmark
		else if(obj.tabCount == 0 && !obj.isBookmark)
			obj.killGroup(obj);
		
		return tab;
	};
}


TabGroup.lastSelectedTab = null;
TabGroup.lastSelectedTabGroup = null;
TabGroup.tabPropertiesMap = new WeakMap();

TabGroup.properties = function(aTab) {
    return TabGroup.tabPropertiesMap.get(aTab);
};

TabGroup.removeTab = function(aTab) {
    TabGroup.getTabGroupForTab(aTab).removeTab(aTab);    
};

TabGroup.selectTab = function(aTab) {
    TabGroup.getTabGroupForTab(aTab).selectTab(aTab);    
};

TabGroup.setTitleChange = function(aTab) {
    let tabProperties = TabGroup.tabPropertiesMap.get(aTab);
    tabProperties.menuitem.setAttribute('titlechanged', 'true');
    tabProperties.menuitem.setAttribute('label', aTab.label);
    tabProperties.tabGroup.titlechangedCounter.increment();
    tabProperties.titlechanged = true;
};

TabGroup.clearTitleChange = function(aTab) {
    let tabProperties = TabGroup.tabPropertiesMap.get(aTab);
    if(!tabProperties.menuitem.hasAttribute('titlechanged'))
        return;
    
    tabProperties.tabGroup.titlechangedCounter.decrement();
    tabProperties.menuitem.removeAttribute('titlechanged');
    tabProperties.titlechanged = false;
};

TabGroup.setUnread = function(aTab) {
    let tabProperties = TabGroup.tabPropertiesMap.get(aTab);
    tabProperties.menuitem.setAttribute('unread', 'true');
    tabProperties.tabGroup.unreadCounter.increment();
    tabProperties.unread = true;
};

TabGroup.clearUnread = function(aTab) {
    let tabProperties = TabGroup.tabPropertiesMap.get(aTab);
    if(!tabProperties.menuitem.hasAttribute('unread'))
        return;
    
    tabProperties.tabGroup.unreadCounter.decrement();
    tabProperties.menuitem.removeAttribute('unread');
    tabProperties.unread = false;
};


TabGroup.setBusy = function(aTab) {
    let tabProperties = TabGroup.tabPropertiesMap.get(aTab);
    tabProperties.tabGroup.busyCounter.increment();
    tabProperties.menuitem.setAttribute('busy', 'true');
};

TabGroup.clearBusy = function(aTab) {
    let tabProperties = TabGroup.tabPropertiesMap.get(aTab);
    if(!tabProperties.menuitem.hasAttribute('busy'))
        return;

    tabProperties.tabGroup.busyCounter.decrement();
    tabProperties.menuitem.removeAttribute('busy');
};

TabGroup.getTabGroupForTab = function(aTab) {
    return TabGroup.tabPropertiesMap.get(aTab).tabGroup;
};
